import decimal
try:
    # Python 2
    from ConfigParser import ConfigParser
except ImportError:
    # Python 3
    from configparser import ConfigParser
import os

from nose.plugins import Plugin

import tests.helpers as th
from .helpers import cfgpath, clear_db, get_app_lock, release_app_lock

_parser = ConfigParser({
    'server': 'localhost',
    'username': 'sa',
    'password': '',
    'database': 'tempdb',
    'port': '1433',
    'ipaddress': '127.0.0.1',
    'instance': '',
})

class ConfigPlugin(Plugin):
    enabled = True
    config_section = ''

    def options(self, parser, env=os.environ):
        """Add command-line options for this plugin"""
        env_opt = 'NOSE_WITH_%s' % self.name.upper()

        parser.add_option("--pymssql-section",
                          type="string",
                          default=env.get('PYMSSQL_TEST_CONFIG', 'DEFAULT'),
                          help="The name of the section to use from tests.cfg"
                        )

        Plugin.options(self, parser, env=env)

    def configure(self, options, config):
        """Configure the plugin"""

        _parser.read(cfgpath)
        section = options.pymssql_section

        if not _parser.has_section(section) and section != 'DEFAULT':
            raise ValueError('the tests.cfg file does not have section: %s' % section)

        th.config.server = os.getenv('PYMSSQL_TEST_SERVER') or _parser.get(section, 'server')
        th.config.user = os.getenv('PYMSSQL_TEST_USERNAME') or _parser.get(section, 'username')
        th.config.password = os.getenv('PYMSSQL_TEST_PASSWORD') or _parser.get(section, 'password')
        th.config.database = os.getenv('PYMSSQL_TEST_DATABASE') or _parser.get(section, 'database')
        th.config.port = os.getenv('PYMSSQL_TEST_PORT') or _parser.get(section, 'port')
        th.config.ipaddress = os.getenv('PYMSSQL_TEST_IPADDRESS') or _parser.get(section, 'ipaddress')
        th.config.instance = os.getenv('PYMSSQL_TEST_INSTANCE') or _parser.get(section, 'instance')
        th.config.orig_decimal_prec = decimal.getcontext().prec

    def begin(self):
        get_app_lock()
        clear_db()

    def finalize(self, result):
        release_app_lock()



from __future__ import with_statement
from os import path, makedirs, environ
import re
import shutil
try:
    import unittest2 as unittest
except ImportError:
    import unittest

from nose.plugins.skip import SkipTest

import _mssql

from .helpers import config
server = config.server
username = config.user
password = config.password
database = config.database
port = config.port
ipaddress = config.ipaddress
instance = config.instance

cdir = path.dirname(__file__)
tmpdir = path.join(cdir, 'tmp')
config_dump_path = path.join(tmpdir, 'freetds-config-dump.txt')
dump_path = path.join(tmpdir, 'freetds-dump.txt')

def setup_module():
    if not path.isdir(tmpdir):
        makedirs(tmpdir)

class TestCons(unittest.TestCase):
    def connect(self, **kwargs):
        environ['TDSDUMPCONFIG'] = config_dump_path
        environ['TDSDUMP'] = dump_path
        _mssql.connect(**kwargs)
        with open(config_dump_path, 'r') as fh:
            return fh.read()

    def test_connection_by_dns_name(self):
        cdump = self.connect(server=server, port=port, user=username, password=password)
        dump_server_name = re.search('server_name = (\S+)', cdump).groups()[0]
        self.assertIn(server, dump_server_name)
        dump_server_host_name = re.search('server_host_name = (\S+)', cdump).groups()[0]
        self.assertEqual(dump_server_host_name, server)
        dump_user_name = re.search('user_name = (\S+)', cdump).groups()[0]
        self.assertEqual(dump_user_name, username)
        dump_port = re.search('port = (\S+)', cdump).groups()[0]
        self.assertIn(port, dump_port)

    def test_connection_by_ip(self):
        cdump = self.connect(server=ipaddress, port=port, user=username, password=password)
        dump_server_name = re.search('server_name = (\S+)', cdump).groups()[0]
        self.assertIn(ipaddress, dump_server_name)
        dump_server_host_name = re.search('server_host_name = (\S+)', cdump).groups()[0]
        self.assertEqual(dump_server_host_name, ipaddress)

    def test_port_override_ipaddress(self):
        server_join = '%s:%s' % (ipaddress, port)
        cdump = self.connect(server=server_join, user=username, password=password)
        dump_server_name = re.search('server_name = (\S+)', cdump).groups()[0]
        self.assertIn(server_join, dump_server_name)
        dump_server_host_name = re.search('server_host_name = (\S+)', cdump).groups()[0]
        self.assertEqual(dump_server_host_name, ipaddress)
        dump_port = re.search('port = (\S+)', cdump).groups()[0]
        self.assertIn(port, dump_port)

    def test_port_override_name(self):
        server_join = '%s:%s' % (server, port)
        cdump = self.connect(server=server_join, user=username, password=password)
        dump_server_name = re.search('server_name = (\S+)', cdump).groups()[0]
        self.assertIn(server, dump_server_name)
        dump_server_host_name = re.search('server_host_name = (\S+)', cdump).groups()[0]
        self.assertEqual(dump_server_host_name, server)
        dump_port = re.search('port = (\S+)', cdump).groups()[0]
        self.assertIn(port, dump_port)

    def test_instance(self):
        if not instance:
            raise SkipTest
        server_join = r'%s\%s' % (server, instance)
        cdump = self.connect(server=server_join, user=username, password=password)
        dump_server_name = re.search('server_name = (\S+)', cdump).groups()[0]
        self.assertIn(server, dump_server_name)
        dump_server_host_name = re.search('server_host_name = (\S+)', cdump).groups()[0]
        self.assertEqual(dump_server_host_name, server)
        dump_port = re.search('port = (\S+)', cdump).groups()[0]
        self.assertEqual(dump_port, 0)



from nose.plugins.skip import SkipTest
from nose.tools import eq_

import _mssql

class TestParameterSub(object):

    def test_single_param(self):
        res = _mssql.substitute_params('SELECT * FROM employees WHERE id=%s', 13)
        eq_(res, b'SELECT * FROM employees WHERE id=13')

        res = _mssql.substitute_params('SELECT * FROM empl WHERE name=%s', b'John Doe')
        eq_(res, b"SELECT * FROM empl WHERE name='John Doe'")

        res = _mssql.substitute_params('SELECT * FROM empl WHERE name=%s', u'John Doe')
        eq_(res, b"SELECT * FROM empl WHERE name=N'John Doe'")

    def test_param_quote(self):
        res = _mssql.substitute_params('SELECT * FROM empl WHERE name=%s', b"John's Doe")
        eq_(res, b"SELECT * FROM empl WHERE name='John''s Doe'")

        res = _mssql.substitute_params('SELECT * FROM empl WHERE name=%s', u"John's Doe")
        eq_(res, b"SELECT * FROM empl WHERE name=N'John''s Doe'")

    def test_single_param_with_d(self):
        res = _mssql.substitute_params('SELECT * FROM employees WHERE id=%d', 13)
        eq_(res, b'SELECT * FROM employees WHERE id=13')

    def test_percent_not_touched_with_no_params(self):
        sql = "SELECT COUNT(*) FROM employees WHERE name LIKE 'J%'"
        res = _mssql.substitute_params(sql, None)
        eq_(res, sql)

    def test_tuple_with_in(self):
        res = _mssql.substitute_params('SELECT * FROM empl WHERE id IN %s', ((5, 6),))
        eq_(res, b"SELECT * FROM empl WHERE id IN (5,6)")

        res = _mssql.substitute_params('SELECT * FROM empl WHERE id IN %s', ((b'foo', b'bar'),))
        eq_(res, b"SELECT * FROM empl WHERE id IN ('foo','bar')")

        res = _mssql.substitute_params('SELECT * FROM empl WHERE id IN %s', ((u'foo', u'bar'),))
        eq_(res, b"SELECT * FROM empl WHERE id IN (N'foo',N'bar')")

        # single item
        res = _mssql.substitute_params('SELECT * FROM empl WHERE id IN %s', ((b'foo',),))
        eq_(res, b"SELECT * FROM empl WHERE id IN ('foo')")

        res = _mssql.substitute_params('SELECT * FROM empl WHERE id IN %s', ((u'foo',),))
        eq_(res, b"SELECT * FROM empl WHERE id IN (N'foo')")

    def test_percent_in_param(self):
        res = _mssql.substitute_params('SELECT * FROM empl WHERE name LIKE %s', b'J%')
        eq_(res, b"SELECT * FROM empl WHERE name LIKE 'J%'")

        res = _mssql.substitute_params('SELECT * FROM empl WHERE name LIKE %s', u'J%')
        eq_(res, b"SELECT * FROM empl WHERE name LIKE N'J%'")

    def test_single_dict_params(self):
        res = _mssql.substitute_params(
                'SELECT * FROM cust WHERE salesrep=%(name)s',
                { 'name': b'John Doe'}
            )
        eq_(res, b"SELECT * FROM cust WHERE salesrep='John Doe'")

        res = _mssql.substitute_params(
                'SELECT * FROM cust WHERE salesrep=%(name)s',
                { 'name': u'John Doe'}
            )
        eq_(res, b"SELECT * FROM cust WHERE salesrep=N'John Doe'")

    def test_weird_key_names_dict_params(self):
        res = _mssql.substitute_params(
                'SELECT * FROM cust WHERE salesrep=%(n %s ##ame)s',
                { 'n %s ##ame': b'John Doe'}
            )
        eq_(res, b"SELECT * FROM cust WHERE salesrep='John Doe'")

        res = _mssql.substitute_params(
                'SELECT * FROM cust WHERE salesrep=%(n %s ##ame)s',
                { 'n %s ##ame': u'John Doe'}
            )
        eq_(res, b"SELECT * FROM cust WHERE salesrep=N'John Doe'")

    def test_multi_dict_params(self):
        res = _mssql.substitute_params(
                'SELECT * FROM empl WHERE (name=%(name)s AND city=%(city)s) or supervisor=%(name)s',
                { 'name': b'John Doe', 'city': b'Nowhere' }
            )
        eq_(res, b"SELECT * FROM empl WHERE (name='John Doe' AND city='Nowhere') or supervisor='John Doe'")

        res = _mssql.substitute_params(
                'SELECT * FROM empl WHERE (name=%(name)s AND city=%(city)s) or supervisor=%(name)s',
                { 'name': u'John Doe', 'city': u'Nowhere' }
            )
        eq_(res, b"SELECT * FROM empl WHERE (name=N'John Doe' AND city=N'Nowhere') or supervisor=N'John Doe'")

    def test_single_and_tuple(self):
        res = _mssql.substitute_params(
                'SELECT * FROM cust WHERE salesrep=%s AND id IN %s',
                (b'John Doe', (1, 2, 3))
            )
        eq_(res, b"SELECT * FROM cust WHERE salesrep='John Doe' AND id IN (1,2,3)")

        res = _mssql.substitute_params(
                'SELECT * FROM cust WHERE salesrep=%s AND id IN %s',
                (u'John Doe', (1, 2, 3))
            )
        eq_(res, b"SELECT * FROM cust WHERE salesrep=N'John Doe' AND id IN (1,2,3)")

    def test_bare_percent_position(self):
        res = _mssql.substitute_params('select 5 % %s', 3)
        eq_(res, b"select 5 % 3")

    def test_bare_percent_dict(self):
        res = _mssql.substitute_params('select 5 % %(divisor)s', {'divisor': 3})
        eq_(res, b"select 5 % 3")

    def test_missing_dict_param(self):
        try:
            res = _mssql.substitute_params(
                'SELECT * FROM cust WHERE salesrep=%(name)s',
                { 'foobar': 'John Doe'}
            )
            assert False, 'expected exception b/c dict did not contain replacement value'
        except ValueError as e:
            if 'params dictionary did not contain value for placeholder' not in str(e):
                raise

    def test_too_many_params(self):
        try:
            res = _mssql.substitute_params(
                'SELECT * FROM cust WHERE salesrep=%s and foo=%s',
                ('bar',)
            )
            assert False, 'expected exception b/c too many params in sql'
        except ValueError as e:
            if 'more placeholders in sql than params available' not in str(e):
                raise



import sys
import threading
import time
import unittest

from nose.plugins.skip import SkipTest
from nose.plugins.attrib import attr

import _mssql

from .helpers import mssqlconn

class TestingThread(threading.Thread):

    def run(self):
        self.running = True
        self.exc = None
        try:
            mssql = mssqlconn()
            for i in range(0, 1000):
                mssql.execute_query('SELECT %d', (i,))
                for row in mssql:
                    assert row[0] == i
            mssql.close()
        except Exception as e:
            self.exc = e
        finally:
            self.running = False

class TestingErrorThread(threading.Thread):

    def run(self):
        self.running = True
        self.exc = None
        try:
            mssql = mssqlconn()
            for i in range(0, 1000):
                try:
                    mssql.execute_query('SELECT unknown_column')
                except:
                    pass
            mssql.close()
        except Exception as e:
            self.exc = e
        finally:
            self.running = False

class SprocTestingErrorThread(threading.Thread):

    def run(self):
        self.running = True
        self.exc = None
        try:
            mssql = mssqlconn()
            for i in range(0, 1000):
                try:
                    proc = mssql.init_procedure('pymssqlErrorThreadTest')
                    proc.execute()
                except:
                    pass
            mssql.close()
        except Exception as e:
            self.exc = e
        finally:
            self.running = False

class ThreadedTests(unittest.TestCase):

    @attr('slow')
    def testThreadedUse(self):
        threads = []
        for i in range(0, 50):
            thread = TestingThread()
            thread.start()
            threads.append(thread)

        running = True
        while running:
            sys.stdout.write(".")
            sys.stdout.flush()
            running = False
            for thread in threads:
                if thread.exc:
                    raise thread.exc
                if thread.running:
                    running = True
                    break
            time.sleep(5)

        sys.stdout.write(" ")
        sys.stdout.flush()

    @attr('slow')
    def testErrorThreadedUse(self):
        threads = []
        for i in range(0, 2):
            thread = TestingErrorThread()
            thread.start()
            threads.append(thread)

        running = True
        while running:
            sys.stdout.write(".")
            sys.stdout.flush()
            running = False
            for thread in threads:
                if thread.exc:
                    raise thread.exc
                if thread.running:
                    running = True
                    break
            time.sleep(5)

        sys.stdout.write(" ")
        sys.stdout.flush()

    def testErrorSprocThreadedUse(self):
        spname = 'pymssqlErrorThreadTest'
        mssql = mssqlconn()
        try:
            mssql.execute_non_query("DROP PROCEDURE [dbo].[%s]" % spname)
        except:
            pass
        mssql.execute_non_query("""
        CREATE PROCEDURE [dbo].[%s]
        AS
        BEGIN
            SELECT unknown_column FROM unknown_table;
        END
        """ % spname)

        threads = []
        for i in range(0, 5):
            thread = SprocTestingErrorThread()
            thread.start()
            threads.append(thread)

        try:
            sys.stdout.write(".")
            sys.stdout.flush()
            running = True
            while running:
                running = False
                for thread in threads:
                    if thread.exc:
                        raise thread.exc
                    if thread.running:
                        running = True
                        break
                time.sleep(5)

            sys.stdout.write(" ")
            sys.stdout.flush()
        finally:
            mssql.execute_non_query("DROP PROCEDURE [dbo].[%s]" % spname)
            mssql.close()

suite = unittest.TestSuite()
suite.addTest(unittest.makeSuite(ThreadedTests))

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# setup.py
#
# Copyright (C) 2009 Damien Churchill <damoxc@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.    If not, write to:
#   The Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor
#   Boston, MA    02110-1301, USA.
#

import contextlib
import os
import os.path as osp
import sys
import getpass
import platform

# Hack to prevent stupid TypeError: 'NoneType' object is not callable error on
# exit of python setup.py test in multiprocessing/util.py _exit_function when
# running python setup.py test (see
# http://www.eby-sarna.com/pipermail/peak/2010-May/003357.html)
try:
    import multiprocessing
except ImportError:
    pass

sys.path.append(osp.join(osp.dirname(__file__), '.pyrex'))

try:
    from setuptools import setup, Extension
    from setuptools.command.develop import develop as STDevelopCmd
except ImportError:
    import ez_setup
    ez_setup.use_setuptools()
    from setuptools import setup, Extension
    from setuptools.command.develop import develop as STDevelopCmd

# Work around Setuptools' broken (Cython-unaware) monkeypatching
# to support Pyrex. This monkeypatching makes the Cython step get skipped if
# using setuptools (not distribute) and Pyrex is not installed.
# (from https://github.com/sampsyo/pylastfp/blob/master/setup.py)
# http://code.google.com/p/pymssql/issues/detail?id=14
try:
    import setuptools.dist
except ImportError:
    pass
else:
    Extension.__init__ = setuptools.dist._get_unpatched(setuptools.extension.Extension).__init__

have_c_files = osp.exists('_mssql.c') and osp.exists('pymssql.c')

from distutils import log
from distutils.cmd import Command
from distutils.command.clean import clean as _clean
from distutils import ccompiler
if have_c_files:
    from distutils.command.build_ext import build_ext as _build_ext
else:
    #
    # Force `setup_requires` stuff like Cython to be installed before proceeding
    #
    from setuptools.dist import Distribution
    Distribution(dict(setup_requires='Cython>=0.19.1'))

    from Cython.Distutils import build_ext as _build_ext
import struct

@contextlib.contextmanager
def stdchannel_redirected(stdchannel, dest_filename):
    """
    A context manager to temporarily redirect stdout or stderr

    e.g.:

    with stdchannel_redirected(sys.stderr, os.devnull):
        ...
    """

    try:
        oldstdchannel = os.dup(stdchannel.fileno())
        dest_file = open(dest_filename, 'w')
        os.dup2(dest_file.fileno(), stdchannel.fileno())

        yield
    finally:
        if oldstdchannel is not None:
            os.dup2(oldstdchannel, stdchannel.fileno())
        if dest_file is not None:
            dest_file.close()

def add_dir_if_exists(filtered_dirs, *dirs):
    for d in dirs:
        if osp.exists(d):
            filtered_dirs.append(d)

compiler = ccompiler.new_compiler()

_extra_compile_args = [
    '-DMSDBLIB'
]

ROOT = osp.abspath(osp.dirname(__file__))
WINDOWS = False
SYSTEM = platform.system()

print("setup.py: platform.system() => %r" % SYSTEM)
print("setup.py: platform.architecture() => %r" % (platform.architecture(),))
print("setup.py: platform.linux_distribution() => %r" % (platform.linux_distribution(),))
print("setup.py: platform.libc_ver() => %r" % (platform.libc_ver(),))

# 32 bit or 64 bit system?
BITNESS = struct.calcsize("P") * 8

if sys.platform == 'win32':
    WINDOWS = True
    include_dirs = []
    library_dirs = []
else:
    include_dirs = []
    library_dirs = []

    FREETDS = None

    if sys.platform == 'darwin':
        FREETDS = osp.join(ROOT, 'freetds', 'darwin_%s' % BITNESS)
        print("""setup.py: Detected Darwin/Mac OS X.
    You can install FreeTDS with Homebrew or MacPorts, or by downloading
    and compiling it yourself.

    Homebrew (http://brew.sh/)
    --------------------------
    brew install freetds

    MacPorts (http://www.macports.org/)
    -----------------------------------
    sudo port install freetds
        """)

    if not os.getenv('PYMSSQL_DONT_BUILD_WITH_BUNDLED_FREETDS'):
        if SYSTEM == 'Linux':
            FREETDS = osp.join(ROOT, 'freetds', 'nix_%s' % BITNESS)
        elif SYSTEM == 'FreeBSD':
            print("""setup.py: Detected FreeBSD.
    For FreeBSD, you can install FreeTDS with FreeBSD Ports or by downloading
    and compiling it yourself.
            """)

    if FREETDS and osp.exists(FREETDS) and os.getenv('PYMSSQL_BUILD_WITH_BUNDLED_FREETDS'):
        print('setup.py: Using bundled FreeTDS in %s' % FREETDS)
        include_dirs.append(osp.join(FREETDS, 'include'))
        library_dirs.append(osp.join(FREETDS, 'lib'))
    else:
        print('setup.py: Not using bundled FreeTDS')

    libraries = ['sybdb']

    with stdchannel_redirected(sys.stderr, os.devnull):
        if compiler.has_function('clock_gettime', libraries=['rt']):
            libraries.append('rt')

usr_local = '/usr/local'
if osp.exists(usr_local):
    add_dir_if_exists(
        include_dirs,
        osp.join(usr_local, 'include'),
        osp.join(usr_local, 'include/freetds'),
        osp.join(usr_local, 'freetds/include')
    )
    add_dir_if_exists(
        library_dirs,
        osp.join(usr_local, 'lib'),
        osp.join(usr_local, 'lib/freetds'),
        osp.join(usr_local, 'freetds/lib')
    )

if sys.platform == 'darwin':
    fink = '/sw'
    if osp.exists(fink):
        add_dir_if_exists(include_dirs, osp.join(fink, 'include'))
        add_dir_if_exists(library_dirs, osp.join(fink, 'lib'))

    macports = '/opt/local'
    if osp.exists(macports):
        # some mac ports paths
        add_dir_if_exists(
            include_dirs,
            osp.join(macports, 'include'),
            osp.join(macports, 'include/freetds'),
            osp.join(macports, 'freetds/include')
        )
        add_dir_if_exists(
            library_dirs,
            osp.join(macports, 'lib'),
            osp.join(macports, 'lib/freetds'),
            osp.join(macports, 'freetds/lib')
        )

if sys.platform != 'win32':
    # Windows uses a different piece of code to detect these
    print('setup.py: include_dirs = %r' % include_dirs)
    print('setup.py: library_dirs = %r' % library_dirs)

class build_ext(_build_ext):
    """
    Subclass the Cython build_ext command so it extracts freetds.zip if it
    hasn't already been done.
    """

    def build_extensions(self):
        global library_dirs, include_dirs, libraries

        if WINDOWS:
            # Detect the compiler so we can specify the correct command line switches
            # and libraries
            from distutils.cygwinccompiler import Mingw32CCompiler
            extra_cc_args = []
            # Distutils bug: self.compiler can be a string or a CCompiler
            # subclass instance, see http://bugs.python.org/issue6377
            if isinstance(self.compiler, str):
                compiler = self.compiler
            elif isinstance(self.compiler, Mingw32CCompiler):
                compiler = 'mingw32'
                freetds_dir = 'ming'
                extra_cc_args = [
                    '-Wl,-allow-multiple-definition',
                    '-Wl,-subsystem,windows-mthreads',
                    '-mwindows',
                    '-Wl,--strip-all'
                ]
                libraries = [
                    'libiconv', 'iconv',
                    'sybdb',
                    'ws2_32', 'wsock32', 'kernel32',
                ]
            else:
                compiler = 'msvc'
                freetds_dir = 'vs2008'
                libraries = [
                    'db-lib', 'tds',
                    'ws2_32', 'wsock32', 'kernel32', 'shell32',
                ]

            FREETDS = osp.join(ROOT, 'freetds', '{0}_{1}'.format(freetds_dir, BITNESS))
            for e in self.extensions:
                e.extra_compile_args.extend(extra_cc_args)
                e.libraries.extend(libraries)
                e.include_dirs.append(osp.join(FREETDS, 'include'))
                e.library_dirs.append(osp.join(FREETDS, 'lib'))

        else:
            for e in self.extensions:
                e.libraries.extend(libraries)
        _build_ext.build_extensions(self)

class clean(_clean):
    """
    Subclass clean so it removes all the Cython generated C files.
    """

    def run(self):
        _clean.run(self)
        for ext in self.distribution.ext_modules:
            cy_sources = [s for s in ext.sources if s.endswith('.pyx')]
            for cy_source in cy_sources:
                c_source = cy_source[:-3] + 'c'
                if osp.exists(c_source):
                    log.info('removing %s', c_source)
                    os.remove(c_source)
                so_built = cy_source[:-3] + 'so'
                if osp.exists(so_built):
                    log.info('removing %s', so_built)
                    os.remove(so_built)

class release(Command):
    """
    Setuptools command to run all the required commands to perform
    a release. This acts differently depending on the platform it
    is being run on.
    """

    description = "Run all the commands required for a release."

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        self.username = None
        self.password = None
        self.store = None

        if WINDOWS:
            self.release_windows()
        else:
            self.release_unix()

    def release_windows(self):
        # generate windows source distributions
        sdist = self.distribution.get_command_obj('sdist')
        sdist.formats = 'zip'
        sdist.ensure_finalized()
        sdist.run()

        # generate a windows egg
        self.run_command('bdist_egg')

        # generate windows installers
        bdist = self.reinitialize_command('bdist')
        bdist.formats = 'zip,wininst'
        bdist.ensure_finalized()
        bdist.run()

        (name, version, fullname) = self.get_info()

        self.upload(fullname + '.zip', '%s %s source zipped' % (name, version))
        self.upload(fullname + '.win32.zip', '%s %s win32 zip installer' % (name, version))
        self.upload(fullname + '.win32-py2.6.exe', '%s %s windows installer' % (name, version))
        self.upload(fullname + '-py2.6-win32.egg', '%s %s windows egg' % (name, version))

    def release_unix(self):
        # generate linux source distributions
        sdist = self.distribution.get_command_obj('sdist')
        sdist.formats = 'gztar,bztar'
        sdist.ensure_finalized()
        sdist.run()

        (name, version, fullname) = self.get_info()
        self.upload(fullname + '.tar.gz', '%s %s source gzipped' % (name, version))
        self.upload(fullname + '.tar.bz2', '%s %s source bzipped' % (name, version))

    def get_info(self):
        """
        Return the project name and version
        """
        return (
            self.distribution.get_name(),
            self.distribution.get_version(),
            self.distribution.get_fullname()
        )

    def upload(self, filename, comment):
        from gc_upload import upload

        if self.username is None:
            username = raw_input('Username: ')
            password = getpass.getpass('Password: ')

            if self.store is None:
                store = raw_input('Store credentials for later use? [Y/n]')
                self.store = store in ('', 'y', 'Y')

            if self.store:
                self.username = username
                self.password = password

        else:
            username = self.username
            password = self.password

        filename = osp.join('dist', filename)
        log.info('uploading %s to googlecode', filename)
        (status, reason, url) = upload(filename, 'pymssql', username, password, comment)
        if not url:
            log.error('upload to googlecode failed: %s', reason)

class DevelopCmd(STDevelopCmd):
    def run(self):
        # add in the nose plugin only when we are using the develop command
        self.distribution.entry_points['nose.plugins'] = ['pymssql_config = tests.nose_plugin:ConfigPlugin']
        STDevelopCmd.run(self)

def ext_modules():
    if have_c_files:
        source_extension = 'c'
    else:
        source_extension = 'pyx'

    return [
        Extension('_mssql', ['_mssql.%s' % source_extension],
            extra_compile_args = _extra_compile_args,
            include_dirs = include_dirs,
            library_dirs = library_dirs
        ),
        Extension('pymssql', ['pymssql.%s' % source_extension],
            extra_compile_args = _extra_compile_args,
            include_dirs = include_dirs,
            library_dirs = library_dirs
        ),
    ]

setup(
    name  = 'pymssql',
    version = '2.0.1',
    description = 'DB-API interface to Microsoft SQL Server for Python. (new Cython-based version)',
    long_description = open('README.rst').read() +"\n\n" + open('ChangeLog_highlights.rst').read(),
    author = 'Damien Churchill',
    author_email = 'damoxc@gmail.com',
    maintainer = 'pymssql Google Group',
    maintainer_email = 'pymssql@googlegroups.com',
    license = 'LGPL',
    platforms = 'any',
    keywords = ['mssql', 'SQL Server', 'database', 'DB-API'],
    url = 'https://github.com/pymssql/pymssql',
    cmdclass = {
        'build_ext': build_ext,
        'clean': clean,
        'release': release,
        'develop': DevelopCmd
    },
    classifiers=[
      "Development Status :: 5 - Production/Stable",
      "Intended Audience :: Developers",
      "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
      "Programming Language :: Python",
      "Programming Language :: Python :: 2.6",
      "Programming Language :: Python :: 2.7",
      "Programming Language :: Python :: 3",
      "Programming Language :: Python :: 3.2",
      "Programming Language :: Python :: 3.3",
      "Programming Language :: Python :: Implementation :: CPython",
      "Topic :: Database",
      "Topic :: Database :: Database Engines/Servers",
      "Topic :: Software Development :: Libraries :: Python Modules",
      "Operating System :: Microsoft :: Windows",
      "Operating System :: POSIX",
      "Operating System :: Unix",
    ],
    data_files = [
        ('', ['_mssql.pyx', 'pymssql.pyx'])
    ],
    zip_safe = False,
    tests_require=['nose', 'unittest2'],
    test_suite='nose.collector',
    ext_modules = ext_modules(),

    # don't remove this, otherwise the customization above in DevelopCmd
    # will break.  You can safely add to it though, if needed.
    entry_points = {}

)
